@@ -14,6 +14,15 @@ from pydantic import BaseModel, Field
 ### Models
 
 
+class ResultsDimensions(BaseModel):
+    lat: int = None
+    lon: int = None
+    geometry: int = None
+    height: int
+    extent: int = None
+    time: int
+
+
 class Stats(BaseModel):
     """
     Statistics related to the results.
@@ -45,6 +54,7 @@ class S3ObjectKey(BaseModel):
     etag: str = None
     run_date: datetime
 
+
 class StationBase(BaseModel):
     """
     Contains the base station data.
@@ -70,11 +80,38 @@ class Station(BaseModel):
     geometry: Geometry
     altitude: float = None
     stats: Stats
+    dimensions: ResultsDimensions = None
     results_object_key: List[S3ObjectKey]
     properties: Dict = Field(None, description='Any additional station specific properties.')
     modified_date: datetime = Field(..., description='The modification date of the last edit.')
 
 
+class StationAgg(BaseModel):
+    """
+    Contains the station data of a dataset.
+    """
+    dataset_id: str = Field(..., description='The unique dataset uuid.')
+    station_id: str = Field(..., description='station uuid based on the geometry')
+    ref: str = None
+    name: str = None
+    osm_id: int = None
+    geometry: Geometry
+    altitude: float = None
+    stats: Stats
+    dimensions: ResultsDimensions = None
+    results_object_key: S3ObjectKey
+    properties: Dict = Field(None, description='Any additional station specific properties.')
+    modified_date: datetime = Field(..., description='The modification date of the last edit.')
+
+
+class TimeRange(BaseModel):
+    """
+
+    """
+    from_date: datetime
+    to_date: datetime
+
+
 class Dataset(BaseModel):
     """
     Dataset data.
@@ -96,10 +133,13 @@ class Dataset(BaseModel):
     geometry_type: str = Field(..., description='This describes how the spatial dimensions are stored. Point, Line, Polygon, or Collection. Follows the OGC spatial data types.')
     grouping: str = Field(..., description='This describes how the staions and the associated data are grouped. Either none or blocks.')
     extent: Geometry = Field(None, description='The geographical extent of the datset as a simple rectangular polygon.')
+    time_range: TimeRange = Field(None, description='The maximum time range of the dataset.')
+    spatial_resolution: float = Field(None, description='The spatial resolution in decimal degrees if the spatial_distribution is grid.')
     cf_standard_name: str = Field(None, description='The CF conventions standard name for the parameter.')
     wrf_standard_name: str = Field(None, description='The WRF standard name for the parameter.')
     precision: float = Field(None, description='The decimal precision of the result values.')
     description: str = Field(None, description='Dataset description.')
+    product_description: str = Field(None, description='Overall description of the product if method is simulation.')
     processing_code: int = Field(None, description='The processing code to determine how the input data should be processed.')
     properties: Dict = Field(None, description='Any additional dataset specific properties.')
     modified_date: datetime = Field(None, description='The modification date of the last edit.')
