@@ -658,12 +658,17 @@ def process_datasets(datasets):
     """
 
     """
-    for ht_ds, ds_list in datasets.items():
-        ds_list2 = assign_ds_ids(ds_list)
-        datasets[ht_ds] = ds_list2
+    if isinstance(datasets, dict):
+        for ht_ds, ds_list in datasets.items():
+            ds_list2 = assign_ds_ids(ds_list)
+            datasets[ht_ds] = ds_list2
 
-    dataset_list = []
-    [dataset_list.extend(ds_list) for ht_ds, ds_list in datasets.items()]
+        dataset_list = []
+        [dataset_list.extend(ds_list) for ht_ds, ds_list in datasets.items()]
+    elif isinstance(datasets, list):
+        dataset_list = assign_ds_ids(datasets)
+    else:
+        raise TypeError('datasets must be either a dict or list.')
 
     return dataset_list
 
@@ -845,7 +850,8 @@ def get_station_data_from_xr(data):
     data : xr.Dataset
     """
     vars1 = list(data.variables)
-    dims1 = list(data.dims.keys())
+    dims0 = dict(data.dims)
+    dims1 = list(dims0.keys())
     parameter = [v for v in vars1 if 'dataset_id' in data[v].attrs][0]
     attrs = data[parameter].attrs.copy()
     data_vars = [parameter]
@@ -882,6 +888,8 @@ def get_station_data_from_xr(data):
     # if not 'virtual_station' in stn_data1:
     #     stn_data1['virtual_station'] = False
 
+    stn_data1['dimensions'] = dims0
+
     if props:
         stn_data1['properties'] = props
 
@@ -1034,12 +1042,183 @@ def prepare_results(data_dict, dataset_list, results_data, run_date_key, sum_clo
             ## Update the data_dict
             ds_id = ds_mapping['dataset_id']
 
-            data_dict[ds_id].append(new1)
+            if isinstance(data_dict, dict):
+                data_dict[ds_id].append(new1)
+            elif isinstance(data_dict, list):
+                data_dict.append({ds_id: new1})
+            else:
+                raise TypeError('data_dict must be a dict or a list.')
 
         del ts_data1
         del data2
 
 
+def prepare_results_v02(dataset, results_data, run_date_key, sum_closed='right', other_closed='left', discrete=True, other_attrs=None, other_encoding=None, skip_resampling=False):
+    """
+
+    """
+    tz_str = 'Etc/GMT{0:+}'
+
+    parameter = dataset['parameter']
+
+    if parameter not in results_data:
+        raise ValueError('The parameter ' + str(parameter) + ' is not in the results_data.')
+
+    ## Determine index
+    data_index = tuple(results_data.dims)
+    vars2 = list(results_data.variables)
+    vars3 = [v for v in vars2 if v not in data_index]
+
+    vars_dict = {}
+    for v in vars3:
+        index1 = results_data[v].dims
+        vars_dict[v] = index1
+
+    ancillary_variables = [v for v, i in vars_dict.items() if (i == data_index) and (v != parameter)]
+
+    main_vars = [parameter] + ancillary_variables
+
+    if not 'time' in data_index:
+        raise ValueError('time must be in the data_df index.')
+
+    other_index = [i for i in data_index if i != 'time']
+
+    ## Iterate through each dataset
+    ds_mapping = copy.deepcopy(dataset)
+    properties = ds_mapping.pop('properties')
+    if 'attrs' in properties:
+        attrs = properties['attrs']
+    else:
+        attrs = {}
+    encoding = properties['encoding']
+
+    attrs1 = copy.deepcopy(attrs)
+    attrs1.update({ds_mapping['parameter']: ds_mapping})
+
+    if isinstance(other_attrs, dict):
+        attrs1.update(other_attrs)
+
+    encoding1 = copy.deepcopy(encoding)
+
+    if isinstance(other_encoding, dict):
+        encoding1.update(other_encoding)
+
+    ## Pre-Process data
+    qual_col = 'quality_code'
+    freq_code = ds_mapping['frequency_interval']
+    utc_offset = ds_mapping['utc_offset']
+
+    main_cols = list(data_index)
+    main_cols.extend([parameter])
+
+    ts_data1 = results_data
+
+    # Convert times to local TZ if necessary
+    if (not freq_code in ['T', 'H', '1H']) and (not utc_offset == '0H'):
+        t1 = int(utc_offset.split('H')[0])
+        tz1 = tz_str.format(-t1)
+        ts_data1['time'] = ts_data1['time'].to_index().tz_localize('UTC').tz_convert(tz1).tz_localize(None)
+
+    ## Aggregate data if necessary
+    # Parameter
+    if (freq_code == 'T') or skip_resampling:
+        data2 = ts_data1
+    else:
+        agg_fun = agg_stat_mapping[ds_mapping['aggregation_statistic']]
+
+        ts_data2 = ts_data1[main_vars].to_dataframe()
+
+        if agg_fun == 'sum':
+            data0 = grp_ts_agg(ts_data2[parameter].reset_index(), other_index, 'time', freq_code, agg_fun, closed=sum_closed)
+        else:
+            data0 = grp_ts_agg(ts_data2[parameter].reset_index(), other_index, 'time', freq_code, agg_fun, discrete, closed=other_closed)
+
+        # Ancillary variables
+        av_list = [data0]
+        for av in ancillary_variables:
+            if qual_col == av:
+                ts_data2[qual_col] = pd.to_numeric(ts_data2[qual_col], errors='coerce', downcast='integer')
+                qual1 = grp_ts_agg(ts_data2[qual_col].reset_index(), other_index, 'time', freq_code, 'min')
+                av_list.append(qual1)
+            else:
+                av1 = grp_ts_agg(ts_data2[av].reset_index(), other_index, 'time', freq_code, 'max')
+                av_list.append(av1)
+
+        # Put the data together
+        data1 = pd.concat(av_list, axis=1)
+        data2 = data1.to_xarray()
+        for v in vars3:
+            if v not in data2:
+                data2[v] = ts_data1[v]
+
+        del ts_data2
+        del data1
+
+    # Convert time back to UTC if necessary
+    if (not freq_code in ['T', 'H', '1H']) and (not utc_offset == '0H'):
+        data2['time'] = data2['time'].to_index().tz_localize(tz1).tz_convert('utc').tz_localize(None)
+
+    ## Check if the entire record is nan
+    n_data = int(data2[parameter].notnull().sum())
+
+    del ts_data1
+
+    if n_data > 0:
+
+        ## Package up the netcdf object
+        new1 = package_xarray(data2, parameter, attrs1, encoding1, run_date=run_date_key, compression='zstd')
+
+        del data2
+
+        ## Update the data_dict
+        # ds_id = ds_mapping['dataset_id']
+
+        return new1
+
+    else:
+        return None
+
+
+def stats_for_dataset_metadata(stns):
+    """
+    I need time_range, extent, and if grid the spatial_resolution.
+    """
+    ## spatial resolution
+    lat_dim = int(np.median([s['dimensions']['lat'] for s in stns]))
+
+    type1 = stns[0]['geometry']['type']
+
+    if type1 in ['Polygon', 'Line']:
+        geo = [s['geometry']['coordinates'][0][0][-1] for s in stns]
+    else:
+        geo = [s['geometry']['coordinates'][0][-1] for s in stns]
+
+    geo1 = np.array(geo).round(5)
+    geo1.sort()
+    diff1 = np.diff(geo1)
+    res1 = round(np.median(diff1[diff1 > 0])/lat_dim, 5)
+
+    ## Extent
+    geo = np.array([s['geometry']['coordinates'] for s in stns]).round(5)
+
+    len1 = int(np.prod(geo.shape)/2)
+    geo1 = geo.T.reshape(2, len1)
+    min_lon, min_lat = geo1.min(axis=1)
+    max_lon, max_lat = geo1.max(axis=1)
+
+    extent1 = mapping(box(min_lon, min_lat, max_lon, max_lat))
+
+    ## time range
+    trange1 = np.array([[s['stats']['from_date'], s['stats']['to_date']] for s in stns])
+    mins, maxes = trange1.T
+
+    min_t = min(mins)
+    max_t = max(maxes)
+
+    ## Make the dict output
+    dict1 = {'spatial_resolution': res1, 'extent': extent1, 'time_range': {'from_date': min_t, 'to_date': max_t}}
+
+    return dict1
 
 
 
